//MyPCF8574.cpp
//Code fuer Arduino und Attiny
//Author Retian
//Version 2.1


/*
Ansteuerung des Portexpander PCF8574

MyPCF8574 Name;

Beispiel siehe unter:
http://arduino-projekte.webnode.at/meine-libraries/portexpander-pcf8574/

Funktionen siehe unter:
http://arduino-projekte.webnode.at/meine-libraries/portexpander-pcf8574/funktionen/

*/

#include "Arduino.h"
#include "MyPCF8574.h"


MyPCF8574::MyPCF8574()
{
}


//***************************************************************


void MyPCF8574::init(byte i2cAdd)
{
  _i2cAdd = i2cAdd;
  Wire.begin();

  lastByte = 0x00;
}

//***************************************************************
//Abfrage, ob der Baustein ueber I2C ansprechbar ist

boolean MyPCF8574::isReady()
{
  Wire.beginTransmission(_i2cAdd);
  if(Wire.endTransmission(true) == 0) return(true);
  else return(false);
}

//***************************************************************
//Schreiben eines Byte auf das Port

void MyPCF8574::writeByte(byte sByte)
{
  _sByte = sByte;

  Wire.beginTransmission(_i2cAdd);
  Wire.write(_sByte);
  Wire.endTransmission(true);
  lastByte = _sByte;
}

//***************************************************************
//Schreiben eines einzelnen Bit auf das Port

void MyPCF8574::writeBit(byte pinNumber, bool bitWert)
{
  _pinNumber = pinNumber;
  _bitWert = bitWert;

  byte bitMask = 0x01 << _pinNumber;
  
  if(_bitWert == 1)
  {
    Wire.beginTransmission(_i2cAdd);
    Wire.write(lastByte | bitMask);
    Wire.endTransmission(true);
    lastByte = lastByte | bitMask;
  }

  if(bitWert == 0)
  {
    bitMask = ~bitMask;

    Wire.beginTransmission(_i2cAdd);
    Wire.write(lastByte & bitMask);
    Wire.endTransmission(true);
    lastByte = lastByte & bitMask;
  }
}

//***************************************************************
//Setzen eines einzelnen Bit des Ports auf HIGH

void MyPCF8574::writeBitHigh(byte pinNumber)
{
  _pinNumber = pinNumber;
  byte bitMask = 0x01 << _pinNumber;

  Wire.beginTransmission(_i2cAdd);
  Wire.write(lastByte | bitMask);
  Wire.endTransmission(true);
  lastByte = lastByte | bitMask;
}

//***************************************************************
//Setzen eines einzelnen Bit des Ports auf LOW

void MyPCF8574::writeBitLow(byte pinNumber)
{
  _pinNumber = pinNumber;
  byte bitMask = ~(0x01 << _pinNumber);

  Wire.beginTransmission(_i2cAdd);
  Wire.write(lastByte & bitMask);
  Wire.endTransmission(true);
  lastByte = lastByte & bitMask;
}

//***************************************************************
//Ruecklesen des als Ausgang gesetztes Port

byte MyPCF8574::readBackByte()
{
  Wire.requestFrom(int(_i2cAdd), 1);
  while(Wire.available() == 0);
  byte wert = Wire.read();
  return(wert);
}

//***************************************************************
//Ruecklesen eines als Ausgang gesetzten Bit des Ports

byte MyPCF8574::readBackBit(byte pinNumber)
{
  _pinNumber = pinNumber;
  byte bitMask = (0x01 << _pinNumber);
    
  Wire.requestFrom(int(_i2cAdd), 1);
  while(Wire.available() == 0);
  byte wert = Wire.read();
  wert = ((wert & bitMask) >> _pinNumber);
  return(wert);
}


//***************************************************************
//Negieren des als Ausgang gesetzten Port

void MyPCF8574::toggleByte()
{
  byte statusByte = readBackByte();
  statusByte = ~statusByte;
  writeByte(statusByte);
}

//***************************************************************
//Negieren eines einzelnen als Ausgang gesetzten Bit

void MyPCF8574::toggleBit(byte pinNumber)
{
  _pinNumber = pinNumber;
    
  byte statusBit = readBackBit(_pinNumber);
  if (statusBit == 0) writeBitHigh(_pinNumber);
  else writeBitLow(_pinNumber);
}

//***************************************************************
//Lesen eines Byte

byte MyPCF8574::readByte()
{ 
  //Pins, die als Eingaenge verwendet werden, muessen vor
  //dem Einlesen auf logisch 1 gesetzt werden!
  //(Aktivierung der internen Pullup-Widerstaende!)
  writeByte(0xFF);
  
  Wire.requestFrom(int(_i2cAdd), 1);
  while(Wire.available() == 0);
  byte wert = Wire.read();
  return(wert);
}

//***************************************************************
//Lesen eines einzelnen Bit

byte MyPCF8574::readBit(byte pinNumber)
{
  _pinNumber = pinNumber;
  byte bitMask = (0x01 << _pinNumber);

  //Pins, die als Eingaenge verwendet werden, muessen vor
  //dem Einlesen auf logisch 1 gesetzt werden!
  //(Aktivierung der internen Pullup-Widerstaende!)
  writeBitHigh(_pinNumber);
  
  Wire.requestFrom(int(_i2cAdd), 1);
  while(Wire.available() == 0);
  byte wert = Wire.read();
  wert = ((wert & bitMask) >> _pinNumber);
  return(wert);
}


